/*======================================================================================
  Tffind_util.c release version 1.1
  Various tffind utilities

  Coded by Matt Weirauch (weirauch@soe.ucsc.edu)
  Aug 11, 2003

  For documentation and source code, see  http://globin.cse.psu.edu/ or
					  http://bio.cse.psu.edu/
  ======================================================================================*/

#include "tffind_util.h"


/* Clean up block list. */
void free_block_list( Block_ptr *b )
{
    Block_ptr b_step,			/* pointer to move through block list */
              new_b_step;		/* next block to be processed */

    if (!b)
        return;

    for (b_step = *b; b_step; b_step = new_b_step)
    {
        new_b_step = b_step->next;
        block_free( b_step );
    }
    *b = NULL;
}


/* Clean up item list. */
void free_item_list( struct ITEM **item_head )
{
    int i;
    struct ITEM *item_step,             /* pointer to move through item list */
                *new_item_step;         /* next item to be processed */

    if (!item_head)
        return;

    for (item_step = *item_head; item_step; item_step = new_item_step)
    {
        new_item_step = item_step->next;
        if (item_step->submat)
            for (i = 0; i < 4; ++i)
                ckfree (item_step->submat[i]);
        ckfree( item_step->submat );
        ckfree( item_step );
    }
    *item_head = NULL;
}


/* Clean up match list. */
void free_match_list( struct MATCH **match_head )
{
    struct MATCH *match_step,           /* pointer to move through match list */
                 *new_match_step;       /* next match to be processed */

    if (!match_head)
        return;

    for (match_step = *match_head; match_step; match_step = new_match_step)
    {
        new_match_step = match_step->next;
        ckfree( match_step->positions );
        ckfree( match_step->which_seqs );
        ckfree( match_step->startpt );
        ckfree( match_step->endpt );
        ckfree( match_step->block_nbrs );
        ckfree( match_step->pct );
        ckfree( match_step );
    }
    *match_head = NULL;
}


/* Clean up 2-d char array. */
void free_char_arr( char **arr, int size )
{
    int i;              /* array index */

    if (!arr)
        return;

    for (i = 0; i < size; i++)
        ckfree( arr[i] );
    ckfree( arr );
}


/* Clean up 2-d float array. */
void free_float_arr( float **arr, int size )
{
    int i;              /* array index */

    if (!arr)
        return;

    for (i = 0; i < size; i++)
        ckfree( arr[i] );
    ckfree( arr );
}


/* Clean up 2-d int array. */
void free_int_arr( int **arr, int size )
{
    int i;              /* array index */

    if (!arr)
        return;

    for (i = 0; i < size; i++)
        ckfree( arr[i] );
    ckfree( arr );
}


/* Clean up Contig struct. */
void free_contig( Contig_ptr contig, int size )
{
    int i;              /* array index */

    if (!contig)
        return;

    for (i = 0; i < size; i++)
    {
        ckfree( contig[i].header );
        ckfree( contig[i].chars );
    }
    ckfree( contig );
}


/* Clean up Seq struct. */
void free_seq( Seq_ptr seq, int size )
{
    int i;              /* array index */
    if (!seq)
        return;

    for (i = 0; i < size; i++)
        ckfree( seq[i].name );
    ckfree( seq );
}


/* Open file and check for succes.  If file cannot be opened, return NULL */
FILE *ckopen_nk( const char *name, const char *mode )
{
    FILE *fp;		/* file pointer */

    if ((fp = fopen(name, mode)) == NULL)
        fprintf(stderr, "*** Cannot open %s. ***", name);
    return fp;
}


/* Error function for reporting problems. */
void error( char *msg, char *buf, int type )
{
    char *type_string;          /* type of error for output (Warning or Error) */

    type_string = ckalloc( STR_SIZE );

    strcpy( type_string, type == WARN ? "Warning" : "Error" );
    fprintf(SCREEN, "\n\ntffind: *** %s: %s ***\n\n", type_string, msg);
    fprintf(SCREEN, "%s\n\n", buf);

    ckfree (type_string);
    if (type == KILL)
        exit( 0 );
}


/* Get entry from keyboard.  return (minus eol) in 'string'. */
void get_entry( char **string )
{
    fgets( *string, STR_SIZE, stdin );
    remove_eol( *string );
}


/* Warn user that their menu choice is invalid. */
void print_invalid_menu_choice( char choice )
{
    fprintf(SCREEN, "\n*** INVALID MENU CHOICE (%c). ***\n",
                    choice == '\n' ? ' ' : choice );
    fprintf(SCREEN, "\nPlease enter a valid choice.  ");
}


/* Remove end-of-line after using fgets(). */
void remove_eol( char *s )
{
    s[strlen(s) - 1] = '\0';
}

/*======================================================================================*/
